#========================================================================================
# package      DE_EPAGES::CrossSelling::t::XMLExportImport
# state        private
#========================================================================================
package DE_EPAGES::CrossSelling::t::XMLExportImport;
use base DE_EPAGES::Shop::API::Test;

use Test::More tests => 9;

use strict;
use warnings;

use DE_EPAGES::Core::API::File qw ( DeleteFile );
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo GetCustomerFolder );
use DE_EPAGES::CrossSelling::API::AutoCrossSelling qw (
    UpdateAutoCrossSellingViaScheduler
);
use DE_EPAGES::CrossSelling::API::Table::AutoCrossSelling qw ( DeleteAutoCrossSelling );
use DE_EPAGES::Object::API::Factory qw (
    LoadClassByAlias
    LoadObject
);
use DE_EPAGES::Object::API::XML::ObjectExportDriver qw ( ExportObjects );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw ( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Object::API::XMLStructureImportHandler;
use DE_EPAGES::Order::API::Order qw ( CreateOrderWithContainer );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

use Readonly;
Readonly::Scalar my $AUTOCROSSSELLINGS_XML => 'AutoCrossSellings.xml';
Readonly::Scalar my $FILENAME     => 'ShopExport.xml';

## no critic qw(ProhibitMagicNumbers)
#========================================================================================
# description  This subroutine tests if AutoCrossSelling information is exported
#               and imported successfully via ExportObjects and ImportObjects.
#========================================================================================
sub test {
    my $self = shift;

    # configure the shop
    my $Shop = $self->shop;
    my $ShopAlias = $Shop->alias;
    $Shop->featureMaxValue('Products', 3);
    $Shop->featureMaxValue('AutoCrossSelling', 1);
    $Shop->set({ AutoCrossSellingCollectData => 1 });

    # delete all products
    $_->delete foreach @{ $Shop->get('ProductFolder')->children };

    # add 2 products
    my %hProducts = map { $_ => _InsertProduct($Shop, $_, '10.0') } qw( Product1 Product2 );

    # add auto cross-selling information via an order
    my $CustomerNo = GetNextCustomerNo($Shop);
    my $CustomerFolder = GetCustomerFolder($Shop);
    my $Customer = LoadClassByAlias('Customer')->insertObject({
        Parent => $CustomerFolder,
        Alias  => $CustomerNo,
    });
    my $Order = CreateOrderWithContainer($Customer);
    my $Container = $Order->get('LineItemContainer');
    $hProducts{$_}->addToContainer($Container, { Quantity => 1 }) foreach (keys %hProducts);
    TriggerHook('API_OrderCreated', { Order => $Order });
    UpdateAutoCrossSellingViaScheduler();

    # verify cross selling data exists
    my $Iterator = $Shop->get('AutoCrossSellings');
    is $Iterator->count, 2, 'created cross-selling entries';

    # export shop
    note "exporting shop to $FILENAME";
    ExportObjects($Shop->parent->pathString, $FILENAME, { withdefaults => 1 }, [ $ShopAlias ], \&_TransForm);

    # search for crossselling information in XML export file
    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $Struct = $ImportHandler->getXMLStructure($FILENAME, {});
    ok( $Struct->existsElement("/$ShopAlias/\"AutoCrossSelling{Product->Products/Product1}\""),
        "retrieve crossselling information on $ShopAlias (Product1)" ) or
        diag explain( $Struct->getRootElements() );

    ok( $Struct->existsElement("/$ShopAlias/\"AutoCrossSelling{Product->Products/Product2}\""),
        "retrieve crossselling information on $ShopAlias (Product2)" ) or
        diag explain( $Struct->getRootElements() );

    # delete all cross-selling data
    $Iterator = $Shop->get('AutoCrossSellings');
    DeleteAutoCrossSelling( $_->{ProductID1}, $_->{ProductID2} ) while <$Iterator>;

    # verify cross selling data is deleted
    $Iterator = $Shop->get('AutoCrossSellings');
    is $Iterator->count, 0, 'deleted cross-selling entries';

    # import data from XML file
    note "importing $FILENAME";
    ImportObjects( $FILENAME, $Shop->parent->pathString );

    # compare exported auto-cross-selling to imported data
    $Iterator = $Shop->get('AutoCrossSellings');
    is $Iterator->count, 2, 'imported cross-selling entries';
    while (my $hInfoAutoCrossSelling = <$Iterator>) {
        my $Product = LoadObject($hInfoAutoCrossSelling->{ProductID1});
        delete $hProducts{$Product->alias};
    }
    ok( scalar keys %hProducts == 0,
        'all auto cross-sellings found in the new shop' );

    # Delete auto cross-selling information in the new shop
    note "deleting by $AUTOCROSSSELLINGS_XML";
    DeleteObjects($AUTOCROSSSELLINGS_XML, $Shop->pathString);

    $Iterator = $Shop->get('AutoCrossSellings');
    is $Iterator->count, 1, 'deleted cross-selling entries with XML file';

    my @AutoCrossSellings;
    $Iterator = $Shop->get('AutoCrossSellings');
    while (my $hInfoAutoCrossSelling = <$Iterator>) {
        push @AutoCrossSellings, $hInfoAutoCrossSelling;
    }

    ok( scalar @AutoCrossSellings == 1, "auto cross-selling information deleted from shop $ShopAlias" );

    my $ProductID = $AutoCrossSellings[0]->{ProductID1};
    my $Product1 = LoadObject($ProductID);
    ok( defined $Product1 && $Product1->alias eq 'Product1',
        'auto cross-selling information on product \'Product1\' remained' );

    return;
}


sub _TransForm {
    my ($TagName, $hAttributes) = @_;

    delete $hAttributes->{'GUID'};
    delete $hAttributes->{'CreationDate'};

    return;
}

sub _InsertProduct {
    my ($Shop, $Alias, $Price) = @_;

    my $ProductFolder = $Shop->folder('Products');
    my $NewProduct = $ProductFolder->insertChild('Product', {
       Alias => $Alias,
       Shop  => $Shop,
    });
    $NewProduct->setListPrice('GBP', 0, $Price);

    return $NewProduct;
}

sub cleanup {
    my $self = shift;

    $self->SUPER::cleanup;

    DeleteFile($FILENAME);    # ShopExport.xml
    return;
}

__PACKAGE__->runScript;
