use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DocdataPay::API::SOAP::SOAP;
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::DocdataPay::API::SOAP::MessageBuilder;
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use Readonly;
use Test::More;
use Test::Exception;

Readonly my $ROOTPATH => '/Test';

## no critic ( ProhibitNestedSubs ProhibitPathConcatenation ProhibitMultiplePackages ProhibitPrivateSubroutineUse ProhibitMagicNumbers)

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            test_getWSDL();
            test_sendCreateRequest();
            test_sendStatusRequest();
            test_buildRequestCall();
            test_getPortName();
            test_getEndPoint();

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub test_getWSDL{
    note((caller(0))[3]);
    my $SOAPojb = DE_EPAGES::DocdataPay::API::SOAP::SOAP->new();

    my $WSDLResult;
    lives_ok { $WSDLResult = $SOAPojb->_getWSDL(); } '_getWSDL ran without error';
}

sub test_sendCreateRequest{
    note((caller(0))[3]);
    my $SOAPojb = DE_EPAGES::DocdataPay::API::SOAP::SOAP->new();
    $SOAPojb->{'TransportHook'} = \&_MockCreateSuccessMessageResponse;

    my $Response;
    lives_ok { $Response = $SOAPojb->_sendRequest('create', _getCreateRequestParameters()); } '_sendRequest ran without error';
}

sub _getCreateRequestParameters{
    my $Builder = DE_EPAGES::DocdataPay::API::SOAP::MessageBuilder->new();
    $Builder->{OrderOrBasket} = LoadObjectByPath("$ROOTPATH/ExtraLongTestShopName00000000000000000000000000000000000000000000/Customers/1001/Orders/1005");
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode( 'en' );

    return $Builder->_buildCreateMessage();
}

sub test_sendStatusRequest{
    note((caller(0))[3]);
    my $SOAPojb = DE_EPAGES::DocdataPay::API::SOAP::SOAP->new();
    $SOAPojb->{'TransportHook'} = \&_MockStatusSuccessMessageResponse;

    my $Response;
    lives_ok { $Response = $SOAPojb->_sendRequest('status', _getStatusRequestParameters()); } '_sendRequest ran without error';

}

sub _getStatusRequestParameters{
    my $Builder = DE_EPAGES::DocdataPay::API::SOAP::MessageBuilder->new();
    $Builder->{OrderOrBasket} = LoadObjectByPath("$ROOTPATH/ExtraLongTestShopName00000000000000000000000000000000000000000000/Customers/1001/Orders/1005");
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode( 'en' );

    return $Builder->_buildStatusMessage();
}

sub _MockCreateSuccessMessageResponse($$) {
    my ($request, $trace) = @_;

    HTTP::Response->new(200, 'Constant'
      , [ 'Content-Type' => 'text/xml' ]
      , <<__ANSWER)
<S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/">
    <S:Body>
        <createResponse xmlns="http://www.docdatapayments.com/services/paymentservice/1_0/">
            <createSuccess>
            <success code="SUCCESS">Operation successful.</success>
            <key>2507C8313DEFDA4285B3954B28A3BA92</key>
            </createSuccess>
        </createResponse>
    </S:Body>
</S:Envelope>
__ANSWER

}

sub _MockStatusSuccessMessageResponse($$) {
    my ($request, $trace) = @_;

    HTTP::Response->new(200, 'Constant'
      , [ 'Content-Type' => 'text/xml' ]
      , <<__ANSWER)
<S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/">
    <S:Body>
        <createResponse xmlns="http://www.docdatapayments.com/services/paymentservice/1_0/">
            <createSuccess>
            <success code="SUCCESS">Operation successful.</success>
            <key>2507C8313DEFDA4285B3954B28A3BA92</key>
            </createSuccess>
        </createResponse>
    </S:Body>
</S:Envelope>
__ANSWER

}

sub test_buildRequestCall{
    note((caller(0))[3]);
    my $SOAPojb = DE_EPAGES::DocdataPay::API::SOAP::SOAP->new();

    lives_ok { $SOAPojb->_buildRequestCall('create') } '_buildRequestCall(create) ran without error';
    lives_ok { $SOAPojb->_buildRequestCall('start') } '_buildRequestCall(start) ran without error';
    lives_ok { $SOAPojb->_buildRequestCall('cancel') } '_buildRequestCall(cancel) ran without error';
    lives_ok { $SOAPojb->_buildRequestCall('capture') } '_buildRequestCall(capture) ran without error';
    lives_ok { $SOAPojb->_buildRequestCall('refund') } '_buildRequestCall(refund) ran without error';
    lives_ok { $SOAPojb->_buildRequestCall('status') } '_buildRequestCall(status) ran without error';
    lives_ok { $SOAPojb->_buildRequestCall('statusExtended') } '_buildRequestCall(statusExtended) ran without error';
}

sub test_getPortName{
    note((caller(0))[3]);
    my $SOAPojb = DE_EPAGES::DocdataPay::API::SOAP::SOAP->new();

    my $Result = $SOAPojb->_getPortName();
    my $Expected = 'PaymentServiceSOAP';

    is($Result, $Expected, "getPortName returned correct value -> $Expected");

}

sub test_getEndPoint{
    note((caller(0))[3]);

    #LIVE URL
    my $SOAPojb = DE_EPAGES::DocdataPay::API::SOAP::SOAP->new();

    my $Result;
    lives_ok { $Result = $SOAPojb->_getEndPoint(); } '_getEndPoint ran without error';
    my $Expected = 'https://secure.docdatapayments.com/ps/services/paymentservice/1_0';
    is($Result, $Expected, "getEndPoint returned correct value -> $Expected");

    #TEST URL
    $SOAPojb = DE_EPAGES::DocdataPay::API::SOAP::SOAP->new(1);

    lives_ok { $Result = $SOAPojb->_getEndPoint(); } '_getEndPoint ran without error';
    $Expected = 'https://test.docdatapayments.com/ps/services/paymentservice/1_0';
    is($Result, $Expected, "getEndPoint TEST returned correct value -> $Expected");
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'ImportOrderForSOAPTest.xml', $ROOTPATH );
    return 1;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

RunScript( Sub => \&Main );

## use critic ( ProhibitNestedSubs ProhibitPathConcatenation ProhibitMultiplePackages ProhibitPrivateSubroutineUse ProhibitMagicNumbers)
1;