use strict;
use Test::More tests => 18;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Net qw( SupportedIpVersions );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::WebInterface::API::Config qw (GetRequestRouterMonitors);
use DE_EPAGES::WebInterface::API::MessageCenter;
use DE_EPAGES::WebInterface::API::RequestRouterMonitor;
use DE_EPAGES::WebInterfacePoolDB::API::ASPoolDB qw( RunOnASPoolDB );
use DE_EPAGES::WebInterface::API::ServerConfig qw ( ServerConfig );

RunScript( Sub => sub {
    my $MessageCenter = DE_EPAGES::WebInterface::API::MessageCenter->new(
        AppServerAddress => '127.0.0.1',
        AppServerPort    => '10048',
    );
    is( $MessageCenter->poolName, 'DefaultPool', 'default poolName' );
    is( $MessageCenter->priority, 0, 'default priority' );
    $MessageCenter->connect;
    ok (1,'connect');
    $MessageCenter->sendInitialFreeRequest;
    ok (1,'initial');
    $MessageCenter->sendWorkingRequest;
    ok (1,'busy');
    $MessageCenter->sendMonitorRequest;
    ok (1,'monitor');

    $MessageCenter->sendFinishRequest;
    ok (1,'finish');

    # custom pool and priority values
    $MessageCenter = DE_EPAGES::WebInterface::API::MessageCenter->new(
        AppServerAddress => '127.0.0.1',
        AppServerPort    => '10048',
        PoolName => 'Pool2',
        Priority => -2,
    );
    is( $MessageCenter->poolName, 'Pool2', 'custom poolName' );
    is( $MessageCenter->priority, -2, 'custom priority' ); ## no critic(ProhibitMagicNumbers)

    # must re-connect, because RequestRouter closes the connection after error
    $MessageCenter = DE_EPAGES::WebInterface::API::MessageCenter->new(
        AppServerAddress => '127.0.0.1',
        AppServerPort    => '10048',
        AppServerPoolName => 'DefaultPool',
        GroupName => 'DefaultAS',
    );
    $MessageCenter->connect;

    my $aMC = GetRequestRouterMonitors()->[0];
    die 'No MONITOR port found in WebInterface.conf' unless $aMC;

    my ($ServerName, $MonitorPort) = @$aMC;
    my $Monitor = DE_EPAGES::WebInterface::API::RequestRouterMonitor->new(
        Address => $ServerName,
        Port => $MonitorPort
    );

    my $hConfiguredRRs = ServerConfig()->requestRouters();
    my $aRequestRouters = $Monitor->requestRouters;
    is( ref($aRequestRouters), 'ARRAY', 'requestRouters' );
    is( scalar @$aRequestRouters + 1, scalar keys %$hConfiguredRRs, 'number of request routers' );

    RunOnASPoolDB( sub {
        my ($ASPoolDB) = @_;
        $ASPoolDB->deleteStore('TestStore');
        $ASPoolDB->insertStore('TestStore', 'TestStoreGUID');
        $ASPoolDB->insertSite('TestStore', 'TestSiteGUID', '/Test/TestSite');
        $ASPoolDB->setSiteRequests('TestStore', '/Test/TestSite', [{
                  'Host'=>'www.testsite.com',
                  'URI'=>'/epages/TestSite.'
        }]);

        my $hSite = $ASPoolDB->getInfoSiteByURI('www.testsite.com', '/epages/TestSite.admin');
        is( $hSite->{'GUID'}, 'TestSiteGUID', 'getInfoSiteByURI' );

        $hSite = $ASPoolDB->getSiteByDomainName('www.testsite.com');
        is( $hSite->{'ObjectPath'}, '/Test/TestSite', 'getSiteByDomainName' );
    } );

    # cleanup
    RunOnASPoolDB( sub { shift->deleteStore('TestStore'); } );

    my $SkipIPv6;
    $SkipIPv6 = 'OS doesn\'t support IPv6' unless ElementOf(6, SupportedIpVersions());
    $SkipIPv6 = 'envvar EPAGES_ENABLE_IPv6 not set' unless $ENV{EPAGES_ENABLE_IPv6};

    if ( $SkipIPv6 ) {
      SKIP: { skip(": $SkipIPv6", 5); }
    } else {
        my $MessageCenter2 = DE_EPAGES::WebInterface::API::MessageCenter->new(
            AppServerAddress => '::1',
            AppServerPort    => '10048',
            AppServerPoolName => 'DefaultPool'
            );
        $MessageCenter2->connect;
        ok (1,'connect');
        $MessageCenter2->sendInitialFreeRequest;
        ok (1,'initial');
        $MessageCenter2->sendWorkingRequest;
        ok (1,'busy');
        $MessageCenter2->sendMonitorRequest;
        ok (1,'monitor');
        $MessageCenter2->sendFinishRequest;
        ok (1,'finish');
    }
});
