use Test::Simple tests => 32;
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::WebInterfacePoolDB::API::ASPoolDB qw( RunOnASPoolDB );
use DE_EPAGES::WebInterfacePoolDB::API::Object::ASPoolDB;
use DE_EPAGES::WebInterfacePoolDB::API::XML::ASPoolDBImportHandler qw (ImportPools);
use DE_EPAGES::WebInterfacePoolDB::API::XML::ASPoolDBDeleteHandler qw (DeletePools);

## no critic qw(ProhibitMagicNumbers)

sub Main () {

    Cleanup();

    my $ASPoolDB = DE_EPAGES::WebInterfacePoolDB::API::Object::ASPoolDB->new;
    ok($ASPoolDB, 'ASPoolDB initialized');

    my $dbi = $ASPoolDB->dbi;
    $dbi->enableTransactionLog(1);
    local $DE_EPAGES::Database::API::Connection::CURRENT_DBI = $dbi;

    ok(!$ASPoolDB->existsPool('TestSitePool'), 'pool TestSitePool does not exist yet' );
    ok(!$ASPoolDB->existsPool('TestPool'), 'pool TestPool does not exist yet' );


    $dbi->transaction(sub {
        ImportPools('PoolTest.xml');
    });

    ok($ASPoolDB->existsPool('TestSitePool'), 'pool TestSitePool exists now' );
    ok($ASPoolDB->existsPool('TestPool'), 'pool TestPool exists now' );

    my $hSite = $ASPoolDB->getInfoSiteByURI('myshop.epages.de', '/epages/TestShop.');
    ok($hSite->{'PoolName'} eq 'TestPool', 'uri assigned pool');

    $dbi->transaction(sub {
        ImportPools('PoolMoveShop.xml');
    });
    $hSite = $ASPoolDB->getInfoSiteByURI('myshop.epages.de', '/epages/TestShop.');
    ok($hSite->{'PoolName'} eq 'TestSitePool', 'moved shop');

    $dbi->transaction(sub {
        DeletePools('PoolDelete.xml');
    });
    my $Transactions = $dbi->getTransactions();
    ok(scalar @$Transactions == 38 || scalar @$Transactions == 0, 'count of Transactions ('.(scalar @$Transactions).')');

    # pool
    ok(!$ASPoolDB->existsPool('TestPool'), 'pool TestPool does not exist anymore' );

    # store
    ok( !$ASPoolDB->existsStore('TestStore'), 'not existsStore' );
    $ASPoolDB->insertStore( 'TestStore', 'TestStoreGUID' );
    ok( $ASPoolDB->existsStore('TestStore'), 'insertStore' );
    ok( $ASPoolDB->existsSite('TestStoreGUID'), 'existsSite TestStoreGUID' );
    my $hStore = $ASPoolDB->getInfoStore( 'TestStore' );
    ok( $hStore->{Name} eq 'TestStore', 'getInfoStore Name' );
    ok( $hStore->{PoolID} > 0, 'getInfoStore PoolID' );
    $ASPoolDB->setSiteRequests('TestStore', '/', [ {
        Host => 'localhost',
        URI => '/epages/TestStore.'
    }, {
        Host => 'www.provider.com',
        URI => '/epages/TestStore.'
    }]);
    $hSite = $ASPoolDB->getInfoSiteByURI( 'localhost', '/epages/TestStore.admin' );
    ok( $hSite->{GUID} eq 'TestStoreGUID', 'getInfoSiteByURI localhost' );
    $hSite = $ASPoolDB->getInfoSiteByURI( 'www.provider.com', '/epages/TestStore.admin' );
    ok( $hSite->{GUID} eq 'TestStoreGUID', 'getInfoSiteByURI www.provider.com' );
    ok( !defined $ASPoolDB->getSiteByDomainName('localhost'), 'no vanity domain: localhost' );
    ok( !defined $ASPoolDB->getSiteByDomainName('www.provider.com'), 'no vanity domain: www.provider.com' );

    # site
    $ASPoolDB->insertSite('TestStore', 'TestSiteGUID', '/Shops/TestSite');
    ok( $ASPoolDB->existsSite('TestSiteGUID'), 'existsSite TestSiteGUID' );
    $ASPoolDB->updateSite('TestSiteGUID', { ObjectPath => '/Shops/TestSite2' });
    $ASPoolDB->setSiteRequests('TestStore', '/Shops/TestSite2', [ {
        Host => 'www.testsite.com',
        URI => '/epages/TestSite.'
    }, {
        Host => 'www.provider.com',
        URI => '/epages/TestSite.'
    }]);
    $hSite = $ASPoolDB->getInfoSiteByURI( 'www.testsite.com', '/epages/TestSite.sf' );
    ok( $hSite->{GUID} eq 'TestSiteGUID', 'getInfoSiteByURI with extension' );
    $hSite = $ASPoolDB->getInfoSiteByURI( 'www.provider.com', '/epages/TestSite.' );
    ok( $hSite->{GUID} eq 'TestSiteGUID', 'getInfoSiteByURI GUID' );
    ok( $hSite->{StoreName} eq 'TestStore', 'getInfoSiteByURI StoreName' );
    ok( $hSite->{ObjectPath} eq '/Shops/TestSite2', 'getInfoSiteByURI ObjectPath' );
    $hSite = $ASPoolDB->getSiteByDomainName('www.testsite.com');
    ok( $hSite->{ObjectPath} eq '/Shops/TestSite2', 'vanity domain: www.testsite.com ObjectPath' );
    ok( $hSite->{StoreAlias} eq 'TestStore', 'vanity domain: www.testsite.com StoreAlias' );
    ok( !defined $ASPoolDB->getSiteByDomainName('localhost'), 'no vanity domain: localhost' );
    ok( !defined $ASPoolDB->getSiteByDomainName('www.provider.com'), 'no vanity domain: www.provider.com' );
    $ASPoolDB->deleteSite('TestSiteGUID');
    ok( !$ASPoolDB->existsSite('TestSiteGUID'), 'deleteSite' );

    ok( !RunOnASPoolDB( sub { shift->existsSite('TestSiteGUID') }), 'RunOnASPoolDB existsSite' );
    ok( RunOnASPoolDB( sub { shift->existsStore('TestStore') }), 'RunOnASPoolDB existsStore' );
    RunOnASPoolDB( sub {
        my $ASPoolDB2 = shift;
        $ASPoolDB2->insertSite('TestStore', 'TestSiteGUID', '/Shops/TestSite');
        ok( $ASPoolDB2->existsSite('TestSiteGUID'), 'RunOnASPoolDB insertSite' );

        $ASPoolDB2->deleteSite('TestSiteGUID');
        ok( !$ASPoolDB2->existsSite('TestSiteGUID'), 'RunOnASPoolDB deleteSite' );
    });


    Cleanup();

    # avoid error on destroy dbi
    $dbi->resetTransactions();
}

sub Cleanup() {
    RunOnStore( Store => 'ASPoolDB', Sub => sub {
        DeletePools('PoolDelete.xml');
    });

    my $ASPoolDB = DE_EPAGES::WebInterfacePoolDB::API::Object::ASPoolDB->new;
    $ASPoolDB->deleteStore( 'TestStore' );
}

RunScript(
    Sub => \&Main
);
