package DE_EPAGES::WebService::t::HelloResource;
use base DE_EPAGES::Database::API::Test;

use strict;
use Test::More tests => 19;
use HTTP::Status qw( HTTP_NOT_ACCEPTABLE HTTP_METHOD_NOT_ALLOWED HTTP_NOT_FOUND );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::WebService::API::Codec::PlainText;
use DE_EPAGES::WebService::API::JSONRestClient;
use DE_EPAGES::WebService::Samples::Server::HelloResource;
use DE_EPAGES::WebService::API::ResourceRegistry;
use DE_EPAGES::WebInterface::API::FakeRequest;

use constant SITE_ALIAS => 'TestSite';
use constant DOMAIN_NAME => 'test-domain';

sub _TestHttpError(&$$) {
    my ( $Sub, $ExpectedStatus, $Message ) = @_;

    eval { $Sub->() };
    ok ExistsError(), "$Message: error expected";
    my $Error = GetError();
    is $Error->code, 'HttpError', "$Message: error code";
    is $Error->vars->{'Code'}, $ExpectedStatus, "$Message: HTTP status code=$ExpectedStatus"
}

sub init {

    DE_EPAGES::WebService::API::ResourceRegistry->add(
        DE_EPAGES::WebService::Samples::Server::HelloResource->new
    );

    my $System = LoadRootObject();
    unless ($System->existsChild(SITE_ALIAS)) {
        my $TestSite = $System->insertChild('Site', { Alias => SITE_ALIAS });
        $TestSite->set({ DomainName => DOMAIN_NAME });
    }
}

sub cleanup {

    my $System = LoadRootObject();
    $System->child(SITE_ALIAS)->delete if $System->existsChild(SITE_ALIAS);
}

sub test {

    my $Resource = DE_EPAGES::WebService::API::JSONRestClient->new('http://'.DOMAIN_NAME.'/api/hello', {
        UserAgent => DE_EPAGES::WebInterface::API::FakeRequest->new,
    });

    _TestHttpError sub { $Resource->get( 'TestID?locale=de_DE' ) }, HTTP_NOT_FOUND, 'empty collection';

    $Resource->put( 'TestID', {
        FirstName => 'Paul',
        LastName => 'Johnson',
    });

    my $hData = $Resource->get('TestID');
    is( $hData->{'FirstName'}, 'Paul', 'put+get' );

    $hData = $Resource->request('GET', 'TestID');
    is( $hData->{'FirstName'}, 'Paul', 'method(GET)' );

    $Resource->delete( 'TestID' );
    _TestHttpError sub { $Resource->get( 'TestID' ) }, HTTP_NOT_FOUND, 'delete';

    my $NewID = $Resource->post({
        FirstName => 'Paula',
        LastName => 'Steward',
    })->{'id'};
    isnt( $NewID, undef, 'post returns id' );
    $hData = $Resource->get( $NewID );
    is( $hData->{'LastName'}, 'Steward', 'post' );

    _TestHttpError sub { $Resource->request('DUMMY', 'TestID') }, HTTP_METHOD_NOT_ALLOWED, 'DUMMY method';

    {
        my $DummyResource = DE_EPAGES::WebService::API::JSONRestClient->new('http://'.DOMAIN_NAME.'/api/dummy', {
            UserAgent => DE_EPAGES::WebInterface::API::FakeRequest->new,
        });
        _TestHttpError sub { $DummyResource->get( 'TestID' ) }, HTTP_NOT_FOUND, 'unknown resource path';
    }

    {
        my $HelloResource = DE_EPAGES::WebService::API::JSONRestClient->new('http://'.DOMAIN_NAME.'/api/hello', {
            UserAgent => DE_EPAGES::WebInterface::API::FakeRequest->new,
            Codec => DE_EPAGES::WebService::API::Codec::PlainText->new,
        });
        _TestHttpError sub { $HelloResource->get( 'TestID' ) }, HTTP_NOT_ACCEPTABLE, 'unknown content-type';
    }
}

__PACKAGE__->runScript;
